@@ -49,12 +49,33 @@
 
 #define KMIXPA_EVENT_KEY "sink-input-by-media-role:event"
 
+
+typedef QMap<uint8_t,Volume::ChannelID> chanIDMap;
+
+struct devinfo
+{
+    int index;
+    int device_index;
+    QString name;
+    QString description;
+    QString icon_name;
+    pa_cvolume volume;
+    pa_channel_map channel_map;
+    bool mute;
+    QString stream_restore_rule;
+
+    Volume::ChannelMask chanMask;
+    chanIDMap chanIDs;
+    unsigned int priority;
+};
+
+
 static unsigned int refcount = 0;
 static pa_context *s_context = NULL;
 static enum { UNKNOWN, ACTIVE, INACTIVE } s_pulseActive = UNKNOWN;
 static int s_outstandingRequests = 0;
 
-QMap<int,Mixer_PULSE*> s_mixers;
+static QMap<int,Mixer_PULSE *> s_mixers;
 
 static devmap outputDevices;
 static devmap captureDevices;
@@ -693,7 +714,7 @@ static void context_state_callback(pa_context *c, void *)
             if (s_mixers.contains(KMIXPA_PLAYBACK)) {
                 qCWarning(KMIX_LOG) << "Connection to PulseAudio daemon closed. Attempting reconnection.";
                 s_pulseActive = UNKNOWN;
-                QTimer::singleShot(50, s_mixers[KMIXPA_PLAYBACK], SLOT(reinit()));
+                QTimer::singleShot(50, s_mixers[KMIXPA_PLAYBACK], &Mixer_PULSE::reinit);
             }
         }
     }
@@ -793,22 +814,22 @@ void Mixer_PULSE::emitControlsReconfigured()
 void Mixer_PULSE::pulseControlsReconfigured()
 {
 	qCDebug(KMIX_LOG) << "Reconfigure " << _mixer->id();
-    ControlManager::instance().announce(_mixer->id(), ControlManager::ControlList, getDriverName());
+    ControlManager::instance()->announce(_mixer->id(), ControlManager::ControlList, getDriverName());
 }
 
-void Mixer_PULSE::pulseControlsReconfigured(QString mixerId)
-{
-	qCDebug(KMIX_LOG) << "Reconfigure " << mixerId;
-    ControlManager::instance().announce(mixerId, ControlManager::ControlList, getDriverName());
-}
+// void Mixer_PULSE::pulseControlsReconfigured(QString mixerId)
+// {
+// 	qCDebug(KMIX_LOG) << "Reconfigure " << mixerId;
+//     ControlManager::instance()->announce(mixerId, ControlManager::ControlList, getDriverName());
+// }
 
-void Mixer_PULSE::updateRecommendedMaster(devmap* map)
+void Mixer_PULSE::updateRecommendedMaster(const devmap *map)
 {
     unsigned int prio = 0;
     shared_ptr<MixDevice> res;
-    MixSet::iterator iter;
 
-    for (iter = m_mixDevices.begin(); iter != m_mixDevices.end(); ++iter) {
+    for (MixSet::iterator iter = m_mixDevices.begin(); iter != m_mixDevices.end(); ++iter)
+    {
         unsigned int devprio = map->value( id2num((*iter)->id()) ).priority;
         if (( devprio > prio ) || !res ) {
             prio = devprio;
@@ -908,8 +929,9 @@ bool Mixer_PULSE::addDevice(devinfo& dev, bool isAppStream)
     v.addVolumeChannels(dev.chanMask);
     setVolumeFromPulse(v, dev);
 
-    MixDevice* md = new MixDevice( _mixer, dev.name, dev.description, dev.icon_name, ms);
+    MixDevice *md = new MixDevice( _mixer, dev.name, dev.description, dev.icon_name, ms);
     if (isAppStream) md->setApplicationStream(true);
+    md->setHardwareId(md->id().toLocal8Bit());
 
     //qCDebug(KMIX_LOG) << "Adding Pulse volume" << dev.name
     //                  << "isCapture" << isCapture
@@ -931,9 +953,9 @@ bool Mixer_PULSE::addDevice(devinfo& dev, bool isAppStream)
     return (true);
 }
 
-Mixer_Backend* PULSE_getMixer( Mixer *mixer, int devnum )
+MixerBackend* PULSE_getMixer( Mixer *mixer, int devnum )
 {
-   Mixer_Backend *l_mixer;
+   MixerBackend *l_mixer;
    l_mixer = new Mixer_PULSE( mixer, devnum );
    return l_mixer;
 }
@@ -963,14 +985,11 @@ bool Mixer_PULSE::connectToDaemon()
 }
 
 
-Mixer_PULSE::Mixer_PULSE(Mixer *mixer, int devnum) : Mixer_Backend(mixer, devnum)
+Mixer_PULSE::Mixer_PULSE(Mixer *mixer, int devnum)
+    : MixerBackend(mixer, devnum)
 {
-    if ( devnum == -1 )
-        m_devnum = 0;
-
-    QString pulseenv = qgetenv("KMIX_PULSEAUDIO_DISABLE");
-    if (pulseenv.toInt())
-        s_pulseActive = INACTIVE;
+    if (devnum==-1) m_devnum = 0;
+    if (qEnvironmentVariableIntValue("KMIX_PULSEAUDIO_DISABLE")) s_pulseActive = INACTIVE;
 
     ++refcount;
     if (INACTIVE != s_pulseActive && 1 == refcount)
@@ -1352,12 +1371,9 @@ void Mixer_PULSE::triggerUpdate()
 
 // Please see KMixWindow::initActionsAfterInitMixer(), it uses the driverName
 
-QString PULSE_getDriverName() {
-        return "PulseAudio";
-}
+const char *PULSE_driverName = "PulseAudio";
 
 QString Mixer_PULSE::getDriverName()
 {
-        return "PulseAudio";
+    return (PULSE_driverName);
 }
-
